@@ -15,13 +15,15 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 """Time series data manipulation, better with pancetta."""
-
 import datetime
 import functools
 import logging
+import math
 import numbers
 import operator
+import random
 import re
+import time
 
 import iso8601
 import lz4
@@ -31,6 +33,11 @@ import six
 
 from gnocchi import utils
 
+# NOTE(sileht): pandas relies on time.strptime()
+# and often triggers http://bugs.python.org/issue7980
+# its dues to our heavy threads usage, this is the workaround
+# to ensure the module is correctly loaded before we use really it.
+time.strptime("2016-02-19", "%Y-%m-%d")
 
 LOG = logging.getLogger(__name__)
 
@@ -45,6 +52,15 @@ class NoDeloreanAvailable(Exception):
             "%s is before %s" % (bad_timestamp, first_timestamp))
 
 
+class BeforeEpochError(Exception):
+    """Error raised when a timestamp before Epoch is used."""
+
+    def __init__(self, timestamp):
+        self.timestamp = timestamp
+        super(BeforeEpochError, self).__init__(
+            "%s is before Epoch" % timestamp)
+
+
 class UnAggregableTimeseries(Exception):
     """Error raised when timeseries cannot be aggregated."""
     def __init__(self, reason):
@@ -80,9 +96,15 @@ class TimeSerie(SerializableMixin):
     def __init__(self, ts=None):
         if ts is None:
             ts = pandas.Series()
-        self.ts = ts[~ts.index.duplicated(keep='last')]
-        if not self.ts.index.is_monotonic:
-            self.ts = self.ts.sort_index()
+        self.ts = self.clean_ts(ts)
+
+    @staticmethod
+    def clean_ts(ts):
+        if ts.index.has_duplicates:
+            ts = ts[~ts.index.duplicated(keep='last')]
+        if not ts.index.is_monotonic:
+            ts = ts.sort_index()
+        return ts
 
     @classmethod
     def from_data(cls, timestamps=None, values=None):
@@ -101,22 +123,17 @@ class TimeSerie(SerializableMixin):
 
     def set_values(self, values):
         t = pandas.Series(*reversed(list(zip(*values))))
-        t = t[~t.index.duplicated(keep='last')]
-        if not t.index.is_monotonic:
-            t = t.sort_index()
-        self.ts = t.combine_first(self.ts)
+        self.ts = self.clean_ts(t).combine_first(self.ts)
 
     def __len__(self):
         return len(self.ts)
 
     @staticmethod
     def _timestamps_and_values_from_dict(values):
-        v = tuple(
-            zip(*dict(
-                (pandas.Timestamp(k), v)
-                for k, v in six.iteritems(values)).items()))
+        timestamps = pandas.to_datetime(list(values.keys()), unit='ns')
+        v = list(values.values())
         if v:
-            return v
+            return timestamps, v
         return (), ()
 
     @classmethod
@@ -144,7 +161,7 @@ class TimeSerie(SerializableMixin):
             return value.nanos / 10e8
 
     @staticmethod
-    def _round_timestamp(ts, freq):
+    def round_timestamp(ts, freq):
         return pandas.Timestamp(
             (pandas.Timestamp(ts).value // freq) * freq)
 
@@ -205,8 +222,8 @@ class BoundTimeSerie(TimeSerie):
 
     def set_values(self, values, before_truncate_callback=None,
                    ignore_too_old_timestamps=False):
+        # NOTE: values must be sorted when passed in.
         if self.block_size is not None and not self.ts.empty:
-            values = sorted(values, key=operator.itemgetter(0))
             first_block_timestamp = self._first_block_timestamp()
             if ignore_too_old_timestamps:
                 for index, (timestamp, value) in enumerate(values):
@@ -250,8 +267,9 @@ class BoundTimeSerie(TimeSerie):
         return basic
 
     def _first_block_timestamp(self):
-        rounded = self._round_timestamp(self.ts.index[-1],
-                                        self.block_size.delta.value)
+        rounded = self.round_timestamp(self.ts.index[-1],
+                                       self.block_size.delta.value)
+
         return rounded - (self.block_size * self.back_window)
 
     def _truncate(self):
@@ -269,8 +287,8 @@ class AggregatedTimeSerie(TimeSerie):
 
     POINTS_PER_SPLIT = 14400
 
-    def __init__(self, sampling, ts=None, max_size=None,
-                 aggregation_method='mean'):
+    def __init__(self, sampling, aggregation_method,
+                 ts=None, max_size=None):
         """A time serie that is downsampled.
 
         Used to represent the downsampled timeserie for a single
@@ -293,17 +311,19 @@ class AggregatedTimeSerie(TimeSerie):
         self.sampling = self._to_offset(sampling).nanos / 10e8
         self.max_size = max_size
         self.aggregation_method = aggregation_method
+        self._truncate(quick=True)
 
     @classmethod
-    def from_data(cls, sampling, timestamps=None, values=None,
-                  max_size=None, aggregation_method='mean'):
-        return cls(ts=pandas.Series(values, timestamps),
-                   max_size=max_size, sampling=sampling,
-                   aggregation_method=aggregation_method)
+    def from_data(cls, sampling, aggregation_method, timestamps=None,
+                  values=None, max_size=None):
+        return cls(sampling=sampling,
+                   aggregation_method=aggregation_method,
+                   ts=pandas.Series(values, timestamps),
+                   max_size=max_size)
 
     @classmethod
     def get_split_key_datetime(cls, timestamp, sampling):
-        return cls._round_timestamp(
+        return cls.round_timestamp(
             timestamp, freq=sampling * cls.POINTS_PER_SPLIT * 10e8)
 
     @staticmethod
@@ -321,23 +341,18 @@ class AggregatedTimeSerie(TimeSerie):
     def split(self):
         groupby = self.ts.groupby(functools.partial(
             self.get_split_key_datetime, sampling=self.sampling))
-        keys = sorted(groupby.groups.keys())
-        for i, ts in enumerate(keys):
-            if i + 1 == len(keys):
-                yield self._split_key_to_string(ts), TimeSerie(self.ts[ts:])
-            elif i + 1 < len(keys):
-                t = self.ts[ts:keys[i + 1]]
-                del t[t.index[-1]]
-                yield self._split_key_to_string(ts), TimeSerie(t)
+        for group, ts in groupby:
+            yield self._split_key_to_string(group), TimeSerie(ts)
 
     @classmethod
-    def from_timeseries(cls, timeseries, sampling, max_size=None,
-                        aggregation_method='mean'):
+    def from_timeseries(cls, timeseries, sampling, aggregation_method,
+                        max_size=None):
         ts = pandas.Series()
         for t in timeseries:
             ts = ts.combine_first(t.ts)
-        return cls(ts=ts, sampling=sampling, max_size=max_size,
-                   aggregation_method=aggregation_method)
+        return cls(sampling=sampling,
+                   aggregation_method=aggregation_method,
+                   ts=ts, max_size=max_size)
 
     def __eq__(self, other):
         return (isinstance(other, AggregatedTimeSerie)
@@ -378,11 +393,11 @@ class AggregatedTimeSerie(TimeSerie):
                 cls._timestamps_and_values_from_dict(d['values']))
 
         return cls.from_data(
+            sampling=sampling,
+            aggregation_method=d.get('aggregation_method', 'mean'),
             timestamps=timestamps,
             values=d.get('values'),
-            max_size=d.get('max_size'),
-            sampling=sampling,
-            aggregation_method=d.get('aggregation_method', 'mean'))
+            max_size=d.get('max_size'))
 
     def to_dict(self):
         if self.ts.empty:
@@ -422,17 +437,18 @@ class AggregatedTimeSerie(TimeSerie):
     def serialize(self):
         return lz4.dumps(msgpack.dumps(self.to_dict()))
 
-    def _truncate(self):
+    def _truncate(self, quick=False):
         """Truncate the timeserie."""
         if self.max_size is not None:
             # Remove empty points if any that could be added by aggregation
-            self.ts = self.ts.dropna()[-self.max_size:]
+            self.ts = (self.ts[-self.max_size:] if quick
+                       else self.ts.dropna()[-self.max_size:])
 
     def _resample(self, after):
         # Group by the sampling, and then apply the aggregation method on
         # the points after `after'
         groupedby = self.ts[after:].groupby(
-            functools.partial(self._round_timestamp,
+            functools.partial(self.round_timestamp,
                               freq=self.sampling * 10e8))
         agg_func = getattr(groupedby, self.aggregation_method_func_name)
         if self.aggregation_method_func_name == 'quantile':
@@ -454,7 +470,7 @@ class AggregatedTimeSerie(TimeSerie):
         if from_timestamp is None:
             from_ = None
         else:
-            from_ = self._round_timestamp(from_timestamp, self.sampling * 10e8)
+            from_ = self.round_timestamp(from_timestamp, self.sampling * 10e8)
         points = self[from_:to_timestamp]
         try:
             # Do not include stop timestamp
@@ -468,9 +484,17 @@ class AggregatedTimeSerie(TimeSerie):
     def update(self, ts):
         if ts.ts.empty:
             return
+        ts.ts = self.clean_ts(ts.ts)
         index = ts.ts.index
         first_timestamp = index[0]
         last_timestamp = index[-1]
+
+        # NOTE(jd) Our whole serialization system is based on Epoch, and we
+        # store unsigned integer, so we can't store anything before Epoch.
+        # Sorry!
+        if first_timestamp.value < 0:
+            raise BeforeEpochError(first_timestamp)
+
         # Build a new time serie excluding all data points in the range of the
         # timeserie passed as argument
         new_ts = self.ts.drop(self.ts[first_timestamp:last_timestamp].index)
@@ -487,9 +511,51 @@ class AggregatedTimeSerie(TimeSerie):
         self._resample(first_timestamp)
         self._truncate()
 
+    @classmethod
+    def benchmark(cls):
+        """Run a speed benchmark!"""
+        points = cls.POINTS_PER_SPLIT
+        sampling = 5
+        compress_times = 50
+
+        now = datetime.datetime(2015, 4, 3, 23, 11)
+
+        for title, values in [
+                ("Simple continuous range", six.moves.range(points)),
+                ("All 0", [float(0)] * points),
+                ("All 1", [float(1)] * points),
+                ("0 and 1", [0, 1] * (points // 2)),
+                ("1 and 0 random",
+                 [random.randint(0, 1)
+                  for x in six.moves.range(points)]),
+                ("Small number random pos/neg",
+                 [random.randint(-100000, 10000)
+                  for x in six.moves.range(points)]),
+                ("Small number random pos",
+                 [random.randint(0, 20000) for x in six.moves.range(points)]),
+                ("Small number random neg",
+                 [random.randint(-20000, 0) for x in six.moves.range(points)]),
+                ("Sin(x)", map(math.sin, six.moves.range(points))),
+                ("random ", [random.random()
+                             for x in six.moves.range(points)]),
+        ]:
+            pts = pandas.Series(values,
+                                [now + datetime.timedelta(seconds=i*sampling)
+                                 for i in six.moves.range(points)])
+            ts = cls(ts=pts, sampling=sampling, aggregation_method='mean')
+            t0 = time.time()
+            for i in six.moves.range(compress_times):
+                s = ts.serialize()
+            t1 = time.time()
+            print(title)
+            print(" Bytes per point: %.2f" % (len(s) / float(points)))
+            print(" Compression speed: %.2f MB/s"
+                  % ((len(msgpack.dumps(ts.to_dict()))
+                      / ((t1 - t0) / compress_times)) / (1024.0 * 1024.0)))
+
     @staticmethod
-    def aggregated(timeseries, from_timestamp=None, to_timestamp=None,
-                   aggregation='mean', needed_percent_of_overlap=100.0):
+    def aggregated(timeseries, aggregation, from_timestamp=None,
+                   to_timestamp=None, needed_percent_of_overlap=100.0):
 
         index = ['timestamp', 'granularity']
         columns = ['timestamp', 'granularity', 'value']
@@ -610,14 +676,13 @@ class TimeSerieArchive(SerializableMixin):
         # Limit the main timeserie to a timespan mapping
         return cls(
             [AggregatedTimeSerie(
-                max_size=size,
                 sampling=sampling,
-                aggregation_method=aggregation_method)
+                aggregation_method=aggregation_method,
+                max_size=size)
              for sampling, size in definitions]
         )
 
-    def fetch(self, from_timestamp=None, to_timestamp=None,
-              timeserie_filter=None):
+    def fetch(self, from_timestamp=None, to_timestamp=None):
         """Fetch aggregated time value.
 
         Returns a sorted list of tuples (timestamp, granularity, value).
@@ -625,8 +690,6 @@ class TimeSerieArchive(SerializableMixin):
         result = []
         end_timestamp = to_timestamp
         for ts in reversed(self.agg_timeseries):
-            if timeserie_filter and not timeserie_filter(ts):
-                continue
             points = ts[from_timestamp:to_timestamp]
             try:
                 # Do not include stop timestamp
@@ -654,3 +717,7 @@ class TimeSerieArchive(SerializableMixin):
     @classmethod
     def from_dict(cls, d):
         return cls([AggregatedTimeSerie.from_dict(a) for a in d['archives']])
+
+
+if __name__ == '__main__':
+    AggregatedTimeSerie.benchmark()
