@@ -24,7 +24,6 @@ from oslotest import base
 from oslotest import mockpatch
 import six
 from six.moves.urllib.parse import unquote
-from stevedore import extension
 try:
     from swiftclient import exceptions as swexc
 except ImportError:
@@ -207,6 +206,10 @@ class FakeRadosModule(object):
             self.kvs_xattrs.pop(key, None)
             self.kvs_omaps.pop(key, None)
 
+        @staticmethod
+        def aio_flush():
+            pass
+
     class FakeRados(object):
         def __init__(self, kvs, kvs_xattrs, kvs_omaps):
             self.kvs = kvs
@@ -332,21 +335,30 @@ class FakeSwiftClient(object):
             raise swexc.ClientException("No such container",
                                         http_status=404)
 
-    def post_account(self, headers, query_string=None, data=None):
-        resp = {'Response Status': '200 OK',
-                'Response Body': '',
-                'Number Deleted': 0,
-                'Number Not Found': 0}
-        if query_string == 'bulk-delete' and data:
-            for path in data.splitlines():
-                try:
-                    __, container, obj = (unquote(path.decode('utf8'))
-                                          .split('/', 2))
-                    del self.kvs[container][obj]
-                    resp['Number Deleted'] += 1
-                except KeyError:
-                    resp['Number Not Found'] += 1
-        return {}, json.dumps(resp).encode('utf-8')
+    def post_account(self, headers, query_string=None, data=None,
+                     response_dict=None):
+        if query_string == 'bulk-delete':
+            resp = {'Response Status': '200 OK',
+                    'Response Body': '',
+                    'Number Deleted': 0,
+                    'Number Not Found': 0}
+            if response_dict is not None:
+                response_dict['status'] = 200
+            if data:
+                for path in data.splitlines():
+                    try:
+                        __, container, obj = (unquote(path.decode('utf8'))
+                                              .split('/', 2))
+                        del self.kvs[container][obj]
+                        resp['Number Deleted'] += 1
+                    except KeyError:
+                        resp['Number Not Found'] += 1
+            return {}, json.dumps(resp).encode('utf-8')
+
+        if response_dict is not None:
+            response_dict['status'] = 204
+
+        return {}, None
 
 
 @six.add_metaclass(SkipNotImplementedMeta)
@@ -375,8 +387,9 @@ class TestCase(base.BaseTestCase):
             return os.path.join(root, project_file)
         return root
 
-    def setUp(self):
-        super(TestCase, self).setUp()
+    @classmethod
+    def setUpClass(self):
+        super(TestCase, self).setUpClass()
         self.conf = service.prepare_service([],
                                             default_config_files=[])
         self.conf.set_override('policy_file',
@@ -395,30 +408,29 @@ class TestCase(base.BaseTestCase):
             self.conf.storage.coordination_url,
             str(uuid.uuid4()).encode('ascii'))
 
-        self.coord.start()
+        self.coord.start(start_heart=True)
 
         with self.coord.get_lock(b"gnocchi-tests-db-lock"):
-            # Force upgrading using Alembic rather than creating the
-            # database from scratch so we are sure we don't miss anything
-            # in the Alembic upgrades. We have a test to check that
-            # upgrades == create but it misses things such as custom CHECK
-            # constraints.
-            self.index.upgrade(nocreate=True,
-                               create_legacy_resource_types=True)
+            self.index.upgrade()
 
         self.coord.stop()
 
         self.archive_policies = self.ARCHIVE_POLICIES.copy()
         self.archive_policies.update(archive_policy.DEFAULT_ARCHIVE_POLICIES)
-        # Used in gnocchi.gendoc
-        if not getattr(self, "skip_archive_policies_creation", False):
-            for name, ap in six.iteritems(self.archive_policies):
-                # Create basic archive policies
-                try:
-                    self.index.create_archive_policy(ap)
-                except indexer.ArchivePolicyAlreadyExists:
-                    pass
+        for name, ap in six.iteritems(self.archive_policies):
+            # Create basic archive policies
+            try:
+                self.index.create_archive_policy(ap)
+            except indexer.ArchivePolicyAlreadyExists:
+                pass
 
+        self.conf.set_override(
+            'driver',
+            os.getenv("GNOCCHI_TEST_STORAGE_DRIVER", "null"),
+            'storage')
+
+    def setUp(self):
+        super(TestCase, self).setUp()
         if swexc:
             self.useFixture(mockpatch.Patch(
                 'swiftclient.client.Connection',
@@ -427,11 +439,6 @@ class TestCase(base.BaseTestCase):
         self.useFixture(mockpatch.Patch('gnocchi.storage.ceph.rados',
                                         FakeRadosModule()))
 
-        self.conf.set_override(
-            'driver',
-            os.getenv("GNOCCHI_TEST_STORAGE_DRIVER", "null"),
-            'storage')
-
         if self.conf.storage.driver == 'file':
             tempdir = self.useFixture(fixtures.TempDir())
             self.conf.set_override('file_basepath',
@@ -446,10 +453,6 @@ class TestCase(base.BaseTestCase):
         # life.
         # self.storage.upgrade(self.index)
 
-        self.mgr = extension.ExtensionManager('gnocchi.aggregates',
-                                              invoke_on_load=True)
-        self.custom_agg = dict((x.name, x.obj) for x in self.mgr)
-
     def tearDown(self):
         self.index.disconnect()
         self.storage.stop()
