@@ -20,14 +20,13 @@ import tempfile
 import threading
 import time
 from unittest import case
-import uuid
 import warnings
 
 from gabbi import fixture
-import sqlalchemy.engine.url as sqlalchemy_url
 import sqlalchemy_utils
 
 from gnocchi import indexer
+from gnocchi.indexer import sqlalchemy
 from gnocchi.rest import app
 from gnocchi import service
 from gnocchi import storage
@@ -102,12 +101,11 @@ class ConfigFixture(fixture.GabbiFixture):
 
         # NOTE(jd) All of that is still very SQL centric but we only support
         # SQL for now so let's say it's good enough.
-        url = sqlalchemy_url.make_url(conf.indexer.url)
-
-        url.database = url.database + str(uuid.uuid4()).replace('-', '')
-        db_url = str(url)
-        conf.set_override('url', db_url, 'indexer')
-        sqlalchemy_utils.create_database(db_url)
+        conf.set_override(
+            'url',
+            sqlalchemy.SQLAlchemyIndexer._create_new_database(
+                conf.indexer.url),
+            'indexer')
 
         index = indexer.get_driver(conf)
         index.connect()
@@ -167,7 +165,9 @@ class MetricdThread(threading.Thread):
 
     def run(self):
         while self.flag:
-            self.storage.process_background_tasks(self.index)
+            metrics = self.storage.list_metric_with_measures_to_process(
+                None, None, full=True)
+            self.storage.process_background_tasks(self.index, metrics)
             time.sleep(0.1)
 
     def stop(self):
